#!/usr/bin/perl -Tw
#####################################################
# gdipinet.pl
#
# This is the GnuDIP (X)INETD server daemon.
#
# See COPYING for licensing information.
#
# Written by:
#
#   Creighton MacDonnell
#   http://macdonnell.ca/
#
# Derived from GnuDIP 2.1.2 written by:
#
#   Mike Machado
#   mike@innercite.com
#
#####################################################

# PERL modules
use strict;
use Socket;

# global variables
use vars qw($conf $gnudipdir);

# locate ourselves
use FindBin;
BEGIN {
  $gnudipdir = '';
  if ($FindBin::Bin =~ /(.*)\/.+?/) {
    $gnudipdir = $1;
  }
}
use lib "$gnudipdir/lib";

# GnuDIP common subroutines
use gdiplib;
use dbusers;

# get preferences from config file
$conf = getconf();
if (! $conf) {
  print STDERR "gdipinet.pl has exited - getconf returned nothing\n";
  exit;
}

# suppress error messages
open (STDERR, ">/dev/null");
#open (STDERR, ">/tmp/gdipinet_STDERR");

# flush after each print
select(STDOUT);
$| = 1;

# send the salt
my $salt = randomsalt();
print STDOUT "$salt\n";

# logger command
my $logger = $$conf{'logger_inet'};

# seconds to wait for response to prompt
my $timeout = $$conf{'timeout'};

# get IP address of remote end
my $client_addr = getpeername(STDIN);
if (! $client_addr) {
  writelog('Could not get IP address of client');
  print "Could not get IP address of client\n";
  exit;
}
my ($port, $packed_ip) = sockaddr_in($client_addr);
my $ip = inet_ntoa($packed_ip);

# only wait $timeout seconds for data before disconnecting
my $sin = '';
vec($sin, fileno(STDIN), 1) = 1;
my $found = select($sin, undef, undef, $timeout);

# timed out?
if (!$found) {
  writelog("Timed out receiving session data from $ip");
  print STDOUT "1\n";
  exit;
}

# get the response
my $data = '';
chomp($data = <STDIN>);
my ($clientuser, $clientpass, $clientdomain, $clientaction, $clientip) = split(/:/, $data);

# got a response?
if ($data eq '') {
  writelog("Empty response from $ip");
  print STDOUT "1\n";
  exit;
}

# sensible request?
if($clientaction ne '0' && $clientaction ne '1' && $clientaction ne '2') {
  writelog("Invalid request from $ip");
  print STDOUT "1\n";
  exit;
}

# retrieve user information
my $userinfo = getuser($clientuser, $clientdomain);

# bad login?
if (!$userinfo                              or
    $clientuser   ne $$userinfo{'username'} or
    $clientdomain ne $$userinfo{'domain'}   or
    $clientpass   ne md5_hex("$$userinfo{'password'}.$salt")
    ) {
  writelog("Invalid login attempt from $ip: user $clientuser.$clientdomain");
  print STDOUT "1\n";
  exit;
}

# client passed an IP address?
if (!defined($clientip) or $clientip eq '') {
  writelog("No IP address passed from $ip: user $clientuser.$clientdomain")
    if $clientaction eq '0';
  $clientip = $ip;
}

# invalid IP address?
if($clientaction ne '1' && !validip($clientip)) {
  writelog(
    "Unserviceable IP address from $ip: user $clientuser.$clientdomain - IP: $clientip");
  print STDOUT "1\n";
  exit;
}

# TTL value
my $TTL = 0;
$TTL = $$conf{'TTL'} if $$conf{'TTL'};
$TTL = $$conf{"TTL.$clientdomain"}
    if $$conf{"TTL.$clientdomain"};

# a modify request?
if ($clientaction eq '0' or $clientaction eq '2') {

  # IP address unchanged?
  if ($$userinfo{'currentip'} eq $clientip) {
    writelog(
      "User $clientuser.$clientdomain remains at ip $clientip");
    updateuser($userinfo);

  # do the update
  } else {
    donsupdate($clientdomain,
      "update delete $clientuser.$clientdomain. A",
      "update add    $clientuser.$clientdomain. $TTL A $clientip");
    writelog(
      "User $clientuser.$clientdomain successful update to ip $clientip");
    $$userinfo{'currentip'} = $clientip;
    updateuser($userinfo);
  }

  if ($clientaction eq '2') {
    print STDOUT "0:$clientip\n";
  } else {
    print STDOUT "0\n";
  }

# an offline request
} else {

  # IP address unchanged?
  if ($$userinfo{'currentip'} eq '0.0.0.0') {
    writelog(
      "User $clientuser.$clientdomain remains removed");
    updateuser($userinfo);

  # do the update
  } else {
    donsupdate($clientdomain,
      "update delete $clientuser.$clientdomain. A");
    writelog(
      "User $clientuser.$clientdomain successful remove from ip $clientip");
    $$userinfo{'currentip'} = '0.0.0.0';
    updateuser($userinfo);
  }

  print STDOUT "2\n";
}

exit;

#####################################################
# subroutines
#####################################################

# call nsupdate
sub donsupdate {
  if (!callnsupdate(@_)) {
    writelog("gdipinet.pl has exited - callnsupdate failed");
    exit;
  }
}

# write to log
sub writelog {
  if (!calllogger($logger, @_)) {
    print STDERR "gdipinet.pl has exited - calllogger failed\n";
    exit;
  }
}

# called for database error
sub dberror {
  exit;
}

